@@ -8,6 +8,7 @@
 """ Unit tests for ‘dput’ module. """
 
 import builtins
+import contextlib
 import doctest
 import hashlib
 import io
@@ -36,7 +37,10 @@ from .helper import (
         FileDouble,
         SubprocessDouble,
         make_fake_file_scenarios,
+        make_fake_locale_scenarios,
         make_unique_slug,
+        patch_locale_environment_variables,
+        patch_locale_getpreferredencoding,
         patch_os_access,
         patch_os_path_exists,
         patch_signal_signal,
@@ -62,63 +66,6 @@ from .test_configfile import (
         set_config,
         )
 
-
-class hexify_string_TestCase(
-        testscenarios.WithScenarios,
-        testtools.TestCase):
-    """ Success test cases for `hexify_string` function. """
-
-    scenarios = [
-            ('empty', {
-                'input_bytes': b"",
-                'expected_result': str(""),
-                }),
-            ('nul', {
-                'input_bytes': b"\x00",
-                'expected_result': str("00"),
-                }),
-            ('ASCII text', {
-                'input_bytes': (
-                    b"Lorem ipsum dolor sit amet"
-                    ),
-                'expected_result': str(
-                    "4c6f72656d20697073756d20646f6c6f722073697420616d6574"),
-                }),
-            ('UTF-8 encoded text', {
-                'input_bytes': b"::".join(text.encode('utf-8') for text in [
-                    "السّلام عليكم",
-                    "⠓⠑⠇⠇⠕",
-                    "你好",
-                    "Hello",
-                    "Γειά σας",
-                    "שלום",
-                    "नमस्ते",
-                    "こんにちは",
-                    "안녕하세요",
-                    "Здра́вствуйте",
-                    "வணக்கம்",
-                    ]),
-                'expected_result': str("3a3a").join([
-                    "d8a7d984d8b3d991d984d8a7d98520d8b9d984d98ad983d985",
-                    "e2a093e2a091e2a087e2a087e2a095",
-                    "e4bda0e5a5bd",
-                    "48656c6c6f",
-                    "ce93ceb5ceb9ceac20cf83ceb1cf82",
-                    "d7a9d79cd795d79d",
-                    "e0a4a8e0a4aee0a4b8e0a58de0a4a4e0a587",
-                    "e38193e38293e381abe381a1e381af",
-                    "ec9588eb8595ed9598ec84b8ec9a94",
-                    "d097d0b4d180d0b0cc81d0b2d181d182d0b2d183d0b9d182d0b5",
-                    "e0aeb5e0aea3e0ae95e0af8de0ae95e0aeaee0af8d",
-                    ]),
-                }),
-            ]
-
-    def test_returns_expected_result_for_input(self):
-        """ Should return expected result for input. """
-        result = dput.dput.hexify_string(self.input_bytes)
-        self.assertEqual(self.expected_result, result)
-
 
 class checksum_test_TestCase(
         testscenarios.WithScenarios,
@@ -213,6 +160,8 @@ class verify_signature_TestCase(
         testtools.TestCase):
     """ Base for test cases for `verify_signature` function. """
 
+    locale_getpreferredencoding_return_value = "UTF-8"
+
     scenarios = NotImplemented
 
     default_args = dict(
@@ -228,6 +177,8 @@ class verify_signature_TestCase(
         super().setUp()
         patch_system_interfaces(self)
 
+        patch_locale_getpreferredencoding(self)
+
         set_config(
                 self,
                 getattr(self, 'config_scenario_name', 'exist-simple'))
@@ -296,7 +247,7 @@ class verify_signature_DebugMessageTestCase(verify_signature_TestCase):
 class verify_signature_ChecksTestCase(verify_signature_TestCase):
     """ Test cases for `verify_signature` checks. """
 
-    scenarios = [
+    file_args_scenarios = [
             ('default', {
                 'expected_checks': ['changes', 'dsc'],
                 }),
@@ -344,10 +295,16 @@ class verify_signature_ChecksTestCase(verify_signature_TestCase):
                 }),
             ]
 
+    locale_scenarios = make_fake_locale_scenarios()
+
+    scenarios = testscenarios.multiply_scenarios(
+            file_args_scenarios, locale_scenarios)
+
     def test_checks_changes_signature_only_when_expected(self):
         """ Should only check the changes document signature if expected. """
         dput.dput.verify_signature(**self.test_args)
-        check_call = unittest.mock.call(self.changes_file_double.fake_file)
+        fake_file = open(self.changes_file_double.path, encoding='UTF-8')
+        check_call = unittest.mock.call(fake_file)
         if 'changes' in self.expected_checks:
             self.assertIn(
                     check_call,
@@ -380,7 +337,8 @@ class verify_signature_ChecksTestCase(verify_signature_TestCase):
     def test_checks_dsc_signature_only_when_expected(self):
         """ Should only check the ‘dsc’ document signature if expected. """
         dput.dput.verify_signature(**self.test_args)
-        check_call = unittest.mock.call(self.dsc_file_double.fake_file)
+        fake_file = open(self.dsc_file_double.path, encoding='UTF-8')
+        check_call = unittest.mock.call(fake_file)
         if 'dsc' in self.expected_checks:
             self.assertIn(
                     check_call,
@@ -440,8 +398,8 @@ class create_upload_file_TestCase(
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
         changes_file_name = os.path.basename(self.changes_file_double.path)
-        self.package_file_name = (
-                os.path.splitext(changes_file_name)[0] + ".source")
+        self.package_file_name = "{name_stem}.source".format(
+                name_stem=os.path.splitext(changes_file_name)[0])
         self.test_args = dict(
                 changes_file_name=self.package_file_name,
                 host=self.test_host,
@@ -485,17 +443,21 @@ class create_upload_file_TestCase(
                 """).format(path=self.upload_log_file_double.path)
         self.assertIn(expected_output, sys.stdout.getvalue())
 
-    def test_opens_log_file_with_expected_mode(self):
-        """ Should open log file with expected write mode. """
+    def test_opens_log_file_with_expected_mode_and_encoding(self):
+        """ Should open log file with expected mode and text encoding. """
         dput.dput.create_upload_file(**self.test_args)
         builtins.open.assert_called_with(
-                self.upload_log_file_double.path, self.expected_open_mode)
+                self.upload_log_file_double.path, self.expected_open_mode,
+                encoding='UTF-8')
 
     def test_writes_expected_message_for_each_file(self):
         """ Should write log message for each upload file path. """
-        with unittest.mock.patch.object(
-                self.upload_log_file_double.fake_file, "close", autospec=True):
+        fake_file = open(
+                self.upload_log_file_double.path, mode='w', encoding='UTF-8')
+        with unittest.mock.patch.object(fake_file, "close", autospec=True):
             dput.dput.create_upload_file(**self.test_args)
+        fake_file.flush()
+        output_text = str(fake_file.buffer.getvalue(), encoding='UTF-8')
         expected_host = self.test_host
         expected_fqdn = self.test_fqdn
         for file_path in self.fake_upload_file_paths:
@@ -505,7 +467,7 @@ class create_upload_file_TestCase(
                         filename=os.path.basename(file_path),
                         fqdn=expected_fqdn, host=expected_host)
             self.expectThat(
-                    self.upload_log_file_double.fake_file.getvalue(),
+                    output_text,
                     testtools.matchers.Contains(expected_message))
 
     def test_calls_sys_exit_if_write_denied(self):
@@ -557,7 +519,7 @@ class run_lintian_test_TestCase(testtools.TestCase):
 
     def patch_lintian_program_file(self):
         """ Patch the Lintian program file for this test case. """
-        file_path = '/usr/bin/lintian'
+        file_path = "/usr/bin/lintian"
         file_double = FileDouble(file_path)
         file_double.register_for_testcase(self)
         self.lintian_program_file_double = file_double
@@ -668,7 +630,7 @@ class dinstall_caller_TestCase(
 
     def patch_ssh_program_file(self):
         """ Patch the SSH program file for this test case. """
-        file_path = '/usr/bin/ssh'
+        file_path = "/usr/bin/ssh"
         file_double = FileDouble(file_path)
         file_double.register_for_testcase(self)
         self.ssh_program_file_double = file_double
@@ -920,7 +882,7 @@ class version_check_TestCase(
 
     def patch_dpkg_program_file(self):
         """ Patch the Dpkg program file for this test case. """
-        file_path = '/usr/bin/dpkg'
+        file_path = "/usr/bin/dpkg"
         file_double = FileDouble(file_path)
         file_double.register_for_testcase(self)
         self.dpkg_program_file_double = file_double
@@ -993,6 +955,42 @@ class version_check_TestCase(
         return subprocess_doubles
 
 
+class version_check_ConsoleEncodingTestCase(version_check_TestCase):
+    """ Test cases for `version_check` with various console text encodings. """
+
+    locale_scenarios = make_fake_locale_scenarios()
+
+    host_architecture_scenarios = [
+            ('host-arch foo', {
+                'host_architecture': "foo",
+                }),
+            ]
+
+    scenarios = testscenarios.multiply_scenarios(
+            locale_scenarios,
+            host_architecture_scenarios,
+            version_check_TestCase.version_scenarios)
+
+    def setUp(self):
+        """ Set up fixtures for this test case. """
+        super().setUp()
+        patch_locale_environment_variables(self)
+
+    def test_emits_debug_message_showing_architecture(self):
+        """ Should emit a debug message for the specified architecture. """
+        test_architecture = self.getUniqueString()
+        subprocess_double = self.dpkg_print_architecture_subprocess_double
+        subprocess_double.set_stdout_content(
+                "{arch}\n".format(arch=test_architecture))
+        self.test_args['debug'] = True
+        with contextlib.suppress(FakeSystemExit):
+            dput.dput.version_check(**self.test_args)
+        expected_output = textwrap.dedent("""\
+                D: detected architecture: '{arch}'
+                """).format(arch=test_architecture)
+        self.assertIn(expected_output, sys.stdout.getvalue())
+
+
 class version_check_ArchitectureMatchTestCase(version_check_TestCase):
     """ Test cases for `version_check` when host architecture matches. """
 
@@ -1026,10 +1024,8 @@ class version_check_ArchitectureMatchTestCase(version_check_TestCase):
         subprocess_double.set_stdout_content(
                 "{arch}\n".format(arch=test_architecture))
         self.test_args['debug'] = True
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.dput.version_check(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = textwrap.dedent("""\
                 D: detected architecture: '{arch}'
                 """).format(arch=test_architecture)
@@ -1049,10 +1045,8 @@ class version_check_ArchitectureMatchTestCase(version_check_TestCase):
         for double in doubles:
             double.set_stderr_content(self.getUniqueString())
         self.test_args['debug'] = False
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.dput.version_check(**self.test_args)
-        except FakeSystemExit:
-            pass
         message_snippet = " stderr output:"
         self.assertNotIn(message_snippet, sys.stdout.getvalue())
 
@@ -1062,10 +1056,8 @@ class version_check_ArchitectureMatchTestCase(version_check_TestCase):
         test_output = self.getUniqueString()
         subprocess_double.set_stderr_content(test_output)
         self.test_args['debug'] = True
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.dput.version_check(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = textwrap.dedent("""\
                 D: dpkg-architecture stderr output: {output!r}
                 """).format(output=test_output)
@@ -1078,10 +1070,8 @@ class version_check_ArchitectureMatchTestCase(version_check_TestCase):
                 ("dpkg", "--field")):
             double.set_stderr_content(test_output)
         self.test_args['debug'] = True
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.dput.version_check(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = textwrap.dedent("""\
                 D: dpkg stderr output: {output!r}
                 """).format(output=test_output)
@@ -1094,10 +1084,8 @@ class version_check_ArchitectureMatchTestCase(version_check_TestCase):
                 ("dpkg", "-s")):
             double.set_stderr_content(test_output)
         self.test_args['debug'] = True
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.dput.version_check(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = textwrap.dedent("""\
                 D: dpkg stderr output: {output!r}
                 """).format(output=test_output)
@@ -1211,7 +1199,7 @@ class version_check_PackageNotInstalledTestCase(version_check_TestCase):
         sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
 
 
-class check_upload_logfile_TestCase(
+class check_upload_logfile_BaseTestCase(
         testscenarios.WithScenarios,
         testtools.TestCase):
     """ Base for test cases for `check_upload_logfile` function. """
@@ -1280,7 +1268,7 @@ class check_upload_logfile_TestCase(
         self.upload_log_file_double = double
 
 
-class check_upload_logfile_SuccessTestCase(check_upload_logfile_TestCase):
+class check_upload_logfile_SuccessTestCase(check_upload_logfile_BaseTestCase):
     """ Success test cases for `check_upload_logfile` function. """
 
     scenarios = [
@@ -1322,14 +1310,12 @@ class check_upload_logfile_SuccessTestCase(check_upload_logfile_TestCase):
 
     def test_returns_expected_result(self):
         """ Should return expected result for the scenario. """
-        try:
+        with contextlib.suppress(FakeSystemExit):
             result = dput.dput.check_upload_logfile(**self.test_args)
-        except FakeSystemExit:
-            pass
         self.assertEqual(self.expected_result, result)
 
 
-class check_upload_logfile_ExitTestCase(check_upload_logfile_TestCase):
+class check_upload_logfile_ExitTestCase(check_upload_logfile_BaseTestCase):
     """ Exit test cases for `check_upload_logfile` function. """
 
     scenarios = [
@@ -1337,10 +1323,39 @@ class check_upload_logfile_ExitTestCase(check_upload_logfile_TestCase):
                 'config_method': "ftp",
                 'config_fqdn': "quux.example.com",
                 'log_content': "foo quux.example.com bar\n",
-                'expected_exit_status': 0,
+                'expected_exit_status': os.EX_OK,
                 }),
             ]
 
+    def test_emits_expected_result_message(self):
+        """ Should emit a message explaining the result of the check. """
+        with contextlib.suppress(FakeSystemExit):
+            dput.dput.check_upload_logfile(**self.test_args)
+        expected_stderr_output = textwrap.dedent("""\
+            Package has already been uploaded to {host} on {fqdn},
+            recorded in upload log file ‘{logfile_path}’.
+            Not attempting upload.
+            """).format(
+                host=self.test_host,
+                fqdn=self.config_fqdn,
+                logfile_path=self.upload_log_file_double.path,
+                )
+        self.assertIn(expected_stderr_output, sys.stderr.getvalue())
+
+    def test_emits_expected_suggestion_message(self):
+        """ Should emit a message suggesting how to override the check. """
+        with contextlib.suppress(FakeSystemExit):
+            dput.dput.check_upload_logfile(**self.test_args)
+        expected_stderr_output = textwrap.dedent("""\
+            To ignore this check and attempt upload anyway,
+            use the ‘--force’ option.
+            """).format(
+                host=self.test_host,
+                fqdn=self.config_fqdn,
+                logfile_path=self.upload_log_file_double.path,
+                )
+        self.assertIn(expected_stderr_output, sys.stderr.getvalue())
+
     def test_calls_sys_exit_with_expected_exit_status(self):
         """ Should call `sys.exit` with expected exit status. """
         with testtools.ExpectedException(FakeSystemExit):
@@ -1348,7 +1363,7 @@ class check_upload_logfile_ExitTestCase(check_upload_logfile_TestCase):
         sys.exit.assert_called_with(self.expected_exit_status)
 
 
-class check_upload_logfile_ErrorTestCase(check_upload_logfile_TestCase):
+class check_upload_logfile_ErrorTestCase(check_upload_logfile_BaseTestCase):
     """ Error test cases for `check_upload_logfile` function. """
 
     log_file_scenarios = [
@@ -1368,7 +1383,7 @@ class check_upload_logfile_ErrorTestCase(check_upload_logfile_TestCase):
         sys.exit.assert_called_with(self.expected_exit_status)
 
 
-# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2024 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
