@@ -8,12 +8,14 @@
 """ Unit tests for upload method behaviour. """
 
 import collections
+import contextlib
 import doctest
 import ftplib
 import getpass
 import http.client
 import importlib
 import io
+import itertools
 import os
 import os.path
 import pkgutil
@@ -23,11 +25,11 @@ import sys
 import tempfile
 import textwrap
 import unittest.mock
+import urllib.parse
+import urllib.request
 
-import httpretty
 import testscenarios
 import testtools
-import urllib.parse
 
 import dput.dput
 import dput.helper.dputhelper
@@ -46,6 +48,7 @@ from .helper import (
         FakeSystemExit,
         FileDouble,
         SubprocessDouble,
+        make_fake_fqdn,
         patch_os_lstat,
         patch_os_stat,
         patch_subprocess_check_call,
@@ -204,7 +207,7 @@ class upload_TestCase(
                 'incoming_path': tempfile.mktemp().lstrip("/"),
                 }),
             ('incoming-has-trailing-slash', {
-                'incoming_path': tempfile.mktemp() + "/",
+                'incoming_path': "{}/".format(tempfile.mktemp()),
                 }),
             ]
 
@@ -365,7 +368,7 @@ class ftp_upload_TestCase(upload_TestCase):
         if not hasattr(self, 'progress_type'):
             self.progress_type = 0
         self.test_args = dict(
-                fqdn=self.getUniqueString(),
+                fqdn=make_fake_fqdn(self),
                 login=self.login,
                 incoming=self.incoming_path,
                 files_to_upload=self.paths_to_upload,
@@ -447,10 +450,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
     def test_emits_error_message_when_ftp_connect_error(self):
         """ Should emit error message when `FTP.connect` raises error. """
         self.ftp_client.connect.side_effect = ftplib.error_temp
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.methods.ftp.upload(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Connection failed, aborting"
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -473,10 +474,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
     def test_emits_error_message_when_ftp_login_permission_error(self):
         """ Should emit error message when `FTP.login` permission error. """
         self.ftp_client.login.side_effect = ftplib.error_perm
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.methods.ftp.upload(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Wrong Password"
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -490,10 +489,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
     def test_emits_error_message_when_ftp_login_eof_error(self):
         """ Should emit error message when `FTP.login` EOF error. """
         self.ftp_client.login.side_effect = EOFError
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.methods.ftp.upload(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Server closed the connection"
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -531,10 +528,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
         """ Should emit error message when destination directory not found. """
         error = ftplib.error_perm("550 Not Found")
         self.ftp_client.cwd.side_effect = error
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.methods.ftp.upload(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Directory to upload to does not exist."
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -579,7 +574,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
         expected_calls = [
                 unittest.mock.call(
                     "STOR {filename}".format(filename=os.path.basename(path)),
-                    registry[path].fake_file, expected_blocksize)
+                    open(path, mode='b'),
+                    expected_blocksize)
                 for path in self.paths_to_upload]
         self.ftp_client.storbinary.assert_has_calls(
                 expected_calls, any_order=True)
@@ -589,8 +585,8 @@ class ftp_upload_NormalFilesTestCase(ftp_upload_TestCase):
         dput.methods.ftp.upload(**self.test_args)
         registry = FileDouble.get_registry_for_testcase(self)
         for path in self.paths_to_upload:
-            fake_file = registry[path].fake_file
-            fake_file.close.assert_called_with()
+            fake_file = open(path, 'b')
+            self.assertTrue(fake_file.closed)
 
 
 class ftp_upload_ErrorTestCase(ftp_upload_TestCase):
@@ -666,10 +662,8 @@ class ftp_upload_ErrorTestCase(ftp_upload_TestCase):
         """ Should emit an error message when `FTP.storbinary` failure. """
         error = ftplib.error_perm("504 Weird Stuff Happened")
         self.ftp_client.storbinary.side_effect = error
-        try:
+        with contextlib.suppress(FakeSystemExit):
             dput.methods.ftp.upload(**self.test_args)
-        except FakeSystemExit:
-            pass
         for path in self.paths_to_upload[:1]:
             expected_output = (
                     "Note: This error might indicate a problem with"
@@ -693,10 +687,8 @@ class ftp_upload_ErrorTestCase(ftp_upload_TestCase):
         for path in self.paths_to_upload:
             double = registry[path]
             double.set_open_scenario('nonexist')
-        try:
+        with contextlib.suppress(EnvironmentError):
             dput.methods.ftp.upload(**self.test_args)
-        except EnvironmentError:
-            pass
         expected_output = (
                 "D: Should exit silently now, but"
                 " will throw exception for debug.")
@@ -744,8 +736,9 @@ def make_expected_filewithprogress_attributes_by_path(testcase, attrs):
     registry = FileDouble.get_registry_for_testcase(testcase)
     for path in testcase.paths_to_upload:
         file_double = registry[path]
+        fake_file = open(path, mode='b')
         expected_attributes = {
-                'f': file_double.fake_file,
+                'f': fake_file,
                 'size': file_double.stat_result.st_size,
                 }
         expected_attributes.update(attrs)
@@ -852,6 +845,58 @@ class ftp_upload_ProgressTestCase(ftp_upload_TestCase):
                     testtools.matchers.Contains(expected_output))
 
 
+def make_mock_httprequest(
+        *,
+        method="GET",
+        url=None,
+        header_fields=None,
+        body=None):
+    """ Make an instrumented mock for `urllib.request.Request`. """
+    request = unittest.mock.Mock(spec=urllib.request.Request)
+    request.url = url
+    request.headers = header_fields
+    if body is None:
+        body = ""
+    body_data = body.encode(dput.methods.http.MESSAGE_BODY_ENCODING)
+    request.data = body_data
+
+    return request
+
+
+reason_by_http_status = {
+    http.client.OK: "Okay",
+    http.client.NON_AUTHORITATIVE_INFORMATION: "Non-Authoritative Information",
+    http.client.INTERNAL_SERVER_ERROR: "Internal Server Error",
+    http.client.UNAUTHORIZED: "Unauthorized",
+}
+
+
+def make_mock_httpresponse(
+        *,
+        version="HTTP/1.1",
+        status=http.client.OK,
+        header_fields=None,
+        body=None):
+    """ Make an instrumented mock for `http.client.HTTPResponse`. """
+    response = unittest.mock.Mock(spec=http.client.HTTPResponse)
+    response.version = version
+    response.status = status
+    response.reason = reason_by_http_status[status]
+
+    if header_fields is None:
+        header_fields = {}
+    response.headers = {name: value for (name, value) in header_fields.items()}
+    if body is None:
+        body = ""
+    body_data = body.encode(dput.methods.http.MESSAGE_BODY_ENCODING)
+    response.msg = body_data
+    response.read.return_value = body_data
+    response.getheader.side_effect = (lambda n: header_fields[n])
+    response.getheaders.return_value = list(header_fields.items())
+
+    return response
+
+
 class http_upload_TestCase(upload_TestCase):
     """ Base for test cases for `methods.http.upload` function. """
 
@@ -880,11 +925,8 @@ class http_upload_TestCase(upload_TestCase):
         """ Set up test fixtures. """
         super().setUp()
 
-        httpretty.enable()
-        self.addCleanup(httpretty.disable)
-
         self.set_response_header_fields()
-        self.patch_put_requests()
+        self.patch_connection()
 
         patch_getpass_getpass(self)
         self.fake_password = self.getUniqueString()
@@ -897,7 +939,7 @@ class http_upload_TestCase(upload_TestCase):
         if not hasattr(self, 'progress_type'):
             self.progress_type = 0
         self.test_args = dict(
-                fqdn=self.getUniqueString(),
+                fqdn=make_fake_fqdn(self),
                 login=self.login,
                 incoming=self.incoming_path,
                 files_to_upload=self.paths_to_upload,
@@ -921,14 +963,44 @@ class http_upload_TestCase(upload_TestCase):
         if not hasattr(self, 'response_header_fields'):
             self.response_header_fields = {}
 
-    def patch_put_requests(self):
-        """ Patch the HTTP PUT requests. """
-        self.path_by_request_uri = {}
-        for path in self.paths_to_upload:
-            upload_uri = self.make_upload_uri(os.path.basename(path))
-            self.path_by_request_uri[upload_uri] = path
-            httpretty.register_uri(
-                    httpretty.PUT, upload_uri, responses=self.responses)
+    def patch_connection(self):
+        """ Patch the connection instance. """
+        mock_connection = unittest.mock.Mock(spec=http.client.HTTPConnection)
+        mock_connection.getresponse.side_effect = itertools.chain(
+                self.responses,
+                itertools.repeat(self.responses[-1]))
+
+        def putrequest(method, url, *, skip_accept_encoding=True):
+            request = make_mock_httprequest(
+                method=method,
+                header_fields={},
+                url=url)
+            mock_connection.latest_request = request
+
+        mock_connection.putrequest = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.putrequest,
+            wraps=putrequest)
+
+        def putheader(header, *values):
+            values = list(values)
+            field_value = "\r\n".join(values)
+            mock_connection.latest_request.headers[header] = field_value
+
+        mock_connection.putheader = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.putheader,
+            wraps=putheader)
+
+        def send(data):
+            mock_connection.latest_request.data = data
+
+        mock_connection.send = unittest.mock.Mock(
+            spec=http.client.HTTPConnection.send,
+            wraps=send)
+        self.mock_connection = mock_connection
+        self.mock_connection_class = unittest.mock.Mock(
+                spec=http.client.HTTPConnection,
+                return_value=self.mock_connection)
+        self.test_args['connection_class'] = self.mock_connection_class
 
 
 class http_upload_SuccessTestCase(http_upload_TestCase):
@@ -937,20 +1009,22 @@ class http_upload_SuccessTestCase(http_upload_TestCase):
     response_scenarios = [
             ('okay', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.OK,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 }),
             ('chatter', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.NON_AUTHORITATIVE_INFORMATION,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 }),
             ]
@@ -1004,7 +1078,7 @@ class http_upload_SuccessTestCase(http_upload_TestCase):
         registry = FileDouble.get_registry_for_testcase(self)
         path = self.paths_to_upload[-1]
         double = registry[path]
-        request = httpretty.last_request()
+        request = self.mock_connection.latest_request
         expected_fields = {
                 'User-Agent': "dput",
                 'Connection': "close",
@@ -1028,10 +1102,12 @@ class http_upload_ProgressTestCase(http_upload_TestCase):
     response_scenarios = [
             ('okay', {
                 'responses': [
-                    httpretty.Response(
-                        status=http.client.OK, body=textwrap.dedent("""\
-                        Lorem ipsum, dolor sit amet.
-                        """)),
+                    make_mock_httpresponse(
+                        status=http.client.OK,
+                        body=textwrap.dedent("""\
+                            Lorem ipsum, dolor sit amet.
+                            """),
+                        ),
                     ],
                 }),
             ]
@@ -1099,10 +1175,8 @@ class http_upload_UnknownProtocolTestCase(http_upload_TestCase):
 
     def test_emits_error_message_when_unknown_protocol(self):
         """ Should emit error message when unknown protocol. """
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Wrong protocol for upload "
         self.assertIn(expected_output, sys.stderr.getvalue())
 
@@ -1146,10 +1220,8 @@ class http_upload_FileStatFailureTestCase(http_upload_TestCase):
 
     def test_emits_error_message(self):
         """ Should emit error message when `os.stat` failure. """
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = textwrap.dedent("""\
                 Determining size of file '{path}' failed
                 """).format(path=self.paths_to_upload[0])
@@ -1171,13 +1243,28 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
             if scenario['paths_to_upload'])
 
     response_scenarios = [
-            ('server-error', {
+            ('server-error body-text', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.INTERNAL_SERVER_ERROR,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
+                    ],
+                'expected_final_status': (
+                    http.client.INTERNAL_SERVER_ERROR,
+                    "Internal Server Error"),
+                'expected_debug_output': textwrap.dedent("""\
+                    Lorem ipsum, dolor sit amet.
+                    """),
+                'expected_exit_status': EXIT_STATUS_FAILURE,
+                }),
+            ('server-error body-empty', {
+                'responses': [
+                    make_mock_httpresponse(
+                        status=http.client.INTERNAL_SERVER_ERROR,
+                        body=""),
                     ],
                 'expected_final_status': (
                     http.client.INTERNAL_SERVER_ERROR,
@@ -1186,21 +1273,30 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
                 }),
             ('authorization-rejected', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
-                    httpretty.Response(
+                            """),
+                        ),
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 'expected_final_status': (
                     http.client.UNAUTHORIZED, "Unauthorized"),
+                'expected_debug_output': textwrap.dedent("""\
+                    Lorem ipsum, dolor sit amet.
+                    """),
                 'expected_exit_status': EXIT_STATUS_FAILURE,
                 }),
             ]
@@ -1216,16 +1312,38 @@ class http_upload_ResponseErrorTestCase(http_upload_TestCase):
         """ Should emit debug message when response status is error. """
         if self.expected_final_status[0] == http.client.UNAUTHORIZED:
             self.skipTest("FIXME: can't see how this ever worked?")
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         (code, reason) = self.expected_final_status
         expected_output = textwrap.dedent("""\
                 Upload failed: {status} {reason}
                 """).format(status=code, reason=reason)
         self.assertIn(expected_output, sys.stdout.getvalue())
 
+    def test_emits_error_response_body_text_when_debug_true(self):
+        """ Should emit error response body text when debug is true. """
+        self.test_args['debug'] = True
+        if not hasattr(self, 'expected_debug_output'):
+            self.skipTest("no response body expected")
+        if self.expected_final_status[0] == http.client.UNAUTHORIZED:
+            self.skipTest("FIXME: can't see how this ever worked?")
+        with contextlib.suppress(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        expected_output = self.expected_debug_output
+        self.assertIn(expected_output, sys.stdout.getvalue())
+
+    def test_emits_no_response_body_text_when_debug_false(self):
+        """ Should emit no response body text when ‘debug’ is false. """
+        self.test_args['debug'] = False
+        if not hasattr(self, 'expected_debug_output'):
+            self.skipTest("no response body expected")
+        if self.expected_final_status[0] == http.client.UNAUTHORIZED:
+            self.skipTest("FIXME: can't see how this ever worked?")
+        with contextlib.suppress(FakeSystemExit):
+            self.function_to_test(**self.test_args)
+        unwanted_output = self.expected_debug_output
+        self.assertNotIn(unwanted_output, sys.stdout.getvalue())
+
     def test_calls_sys_exit_when_response_status_error(self):
         """ Should call `sys.exit` when response status is error. """
         if self.expected_final_status[0] == http.client.UNAUTHORIZED:
@@ -1246,17 +1364,21 @@ class http_upload_ResponseUnauthorizedTestCase(http_upload_TestCase):
     response_scenarios = [
             ('authorization-accepted', {
                 'responses': [
-                    httpretty.Response(
+                    make_mock_httpresponse(
                         status=http.client.UNAUTHORIZED,
-                        www_authenticate="Basic realm=\"pellentesque\"",
+                        header_fields={
+                            'WWW-Authenticate': "Basic realm=\"pellentesque\"",
+                            },
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
-                    httpretty.Response(
+                            """),
+                        ),
+                    make_mock_httpresponse(
                         status=http.client.OK,
                         body=textwrap.dedent("""\
                             Lorem ipsum, dolor sit amet.
-                            """)),
+                            """),
+                        ),
                     ],
                 'expected_final_status': (http.client.OK, "Okay"),
                 'expected_exit_status': EXIT_STATUS_SUCCESS,
@@ -1384,7 +1506,7 @@ class scp_upload_TestCase(ssh_channel_upload_TestCase):
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
         self.test_args = dict(
-                fqdn=self.getUniqueString(),
+                fqdn=make_fake_fqdn(self),
                 login=self.login,
                 incoming=self.incoming_path,
                 files_to_upload=self.paths_to_upload,
@@ -1456,10 +1578,8 @@ class scp_upload_ScpTestCase(scp_upload_TestCase):
         """ Should emit error message when ‘scp’ command fails. """
         double = self.scp_subprocess_double
         double.set_subprocess_check_call_scenario("failure")
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Error while uploading."
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -1518,10 +1638,8 @@ class scp_upload_ChmodTestCase(scp_upload_TestCase):
         """ Should emit error message when ‘ssh … chmod’ command fails. """
         double = self.ssh_chmod_subprocess_double
         double.set_subprocess_check_call_scenario("failure")
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Error while fixing permissions."
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -1549,6 +1667,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         """ Set up test fixtures. """
         super().setUp()
 
+        self.expected_rsync_options = dput.methods.rsync.rsync_options
         self.set_rsync_subprocess_double()
 
         self.expected_ssh_options = []
@@ -1557,7 +1676,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
     def set_test_args(self):
         """ Set the arguments for the test call to the function. """
         self.test_args = dict(
-                fqdn=self.getUniqueString(),
+                fqdn=make_fake_fqdn(self),
                 login=self.login,
                 incoming=self.incoming_path,
                 files_to_upload=self.paths_to_upload,
@@ -1571,9 +1690,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         command_file_path = "/usr/bin/rsync"
         argv = [os.path.basename(command_file_path)]
         argv.extend(self.paths_to_upload)
-        argv.extend([
-                "--copy-links", "--progress", "--partial",
-                "-zave", "ssh -x"])
+        argv.extend(self.expected_rsync_options)
         argv.append(make_remote_spec(
                 username=self.login, host=self.test_args['fqdn'],
                 dir_path=self.incoming_path))
@@ -1606,10 +1723,8 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         """ Should emit error message when ‘rsync’ command fails. """
         double = self.rsync_subprocess_double
         double.set_subprocess_check_call_scenario("failure")
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Error while uploading."
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -1643,10 +1758,8 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         """ Should emit error message when ‘ssh … chmod’ command fails. """
         double = self.ssh_chmod_subprocess_double
         double.set_subprocess_check_call_scenario("failure")
-        try:
+        with contextlib.suppress(FakeSystemExit):
             self.function_to_test(**self.test_args)
-        except FakeSystemExit:
-            pass
         expected_output = "Error while fixing permission."
         self.assertIn(expected_output, sys.stdout.getvalue())
 
@@ -1659,7 +1772,7 @@ class rsync_upload_TestCase(ssh_channel_upload_TestCase):
         sys.exit.assert_called_with(EXIT_STATUS_FAILURE)
 
 
-# Copyright © 2015–2018 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2024 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
